/**
   A simple demonstration of loading and drawing a GRS file.

   The basic structure of a GRS file is:
   1.) A number of comment lines, followed by a line starting with an asterisk, '*'.
   2.) The "extent" of the figure in world coordinates (left, top, right, bottom).
   3.) The number of line-strips (i.e, polylines) in the figure.
   4.) The list of line-strips. Each line-strip starts with the number of vertices in the
       line-strip, followed by the (x, y) world coordinates for each vertex.
*/
import java.util.Scanner;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

public class DrawGRSFile
{
   public static void main(String[] args)
   {
      // Check for a file name on the command line.
      if ( 0 == args.length )
      {
         System.err.println("Usage: java DrawGRSFile <GRS-file-name>");
         System.exit(-1);
      }

      // Open the file named on the command line.
      FileInputStream fis = null;
      try
      {
         fis = new FileInputStream(args[0]);
      }
      catch (FileNotFoundException e)
      {
         e.printStackTrace(System.err);
         System.err.printf("ERROR! Could not open GRS file: %s\n", args[0]);
         System.exit(-1);
      }

      Scanner scanner = new Scanner(fis);

      // skip over the comment lines
      String line = scanner.nextLine();
      while ( ! line.startsWith("*") )
      {
         System.out.println(line);
         line = scanner.nextLine();
      }

      // read the figure extents
      double left   = scanner.nextDouble();
      double top    = scanner.nextDouble();
      double right  = scanner.nextDouble();
      double bottom = scanner.nextDouble();

      // read the number of line-strips
      int numLineStrips = scanner.nextInt();

      // Create a Model object to hold the geometry.
      Model model = new Model();
      model.setColor( java.awt.Color.blue );

      // read each line-strip
      for(int j = 0; j < numLineStrips; j++)
      {
         // read the number of vertices in this line-strip
         int numVertices = scanner.nextInt();

         // put this line-strip in the Model object
         double x = scanner.nextDouble();  // read the first vertex in the line-strip
         double y = scanner.nextDouble();
         Vertex v0 = new Vertex(x, y, 0);
         for (int i = 1; i < numVertices; i++)
         {
            // read the next world coordinate pair
            x = scanner.nextDouble();
            y = scanner.nextDouble();
            Vertex v1 = new Vertex(x, y, 0);
            // create a new LineSegment in the Model
            model.lineSegmentList.add( new LineSegment(v0, v1) );
            v0 = v1;
         }
      }


      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.modelList.add(model);

      // Create a Camera object.
      Camera camera = new Camera();

      // Give the Camera a view matrix.
      camera.view2Identity();

      // Set up the camera's view volume.
      double near   =  1.0;
      double far    =  2.0;
      camera.projFrustum(left, right, bottom, top, near, far);

      // Set up the camera's location and orientation.
      double eyex=0,    eyey=0,    eyez=1;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      camera.viewLookAt(eyex,    eyey,    eyez,
                        centerx, centery, centerz,
                        upx,     upy,     upz);

      // Add the Camera to the Scene.
      scene.setCamera(camera);

      // Create a FrameBuffer to render the Scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.lightGray);
      // Render our scene into the frame buffer.
      SceneRender.renderScene(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File("DrawGRSFile.ppm");
   }
}